@@ -2,7 +2,7 @@
 
 #written by Jeremy M. Beaulieu
 
-corDISC<-function(phy, data, ntraits=2, rate.mat=NULL, model=c("ER","SYM","ARD"), node.states=c("joint", "marginal", "scaled"), p=NULL, root.p=NULL, ip=NULL, lb=0, ub=100, diagn=TRUE){
+corDISC<-function(phy, data, ntraits=2, rate.mat=NULL, model=c("ER","SYM","ARD"), node.states=c("joint", "marginal", "scaled"), p=NULL, root.p=NULL, ip=NULL, lb=0, ub=100, diagn=FALSE){
 	
 	# Checks to make sure node.states is not NULL.  If it is, just returns a diagnostic message asking for value.
 	if(is.null(node.states)){
@@ -46,50 +46,8 @@ corDISC<-function(phy, data, ntraits=2, rate.mat=NULL, model=c("ER","SYM","ARD")
 	nl=2
 
 	#Some pre-processing code in order to print totals to the screen:
-	if(dim(data.sort)[2]==2){
-		x<-data.sort[,1]
-		y<-data.sort[,2]
-		liks.tmp <- matrix(0, nb.tip, nl^k)
-		for(i in 1:nb.tip){
-			if(is.na(x[i])){
-				x[i]=2
-				y[i]=2
-			}
-		}
-		for(i in 1:nb.tip){
-			if(x[i]==0 & y[i]==0){liks.tmp[i,1]=1}
-			if(x[i]==0 & y[i]==1){liks.tmp[i,2]=1}
-			if(x[i]==1 & y[i]==0){liks.tmp[i,3]=1}
-			if(x[i]==1 & y[i]==1){liks.tmp[i,4]=1}
-			if(x[i]==2 & y[i]==2){liks.tmp[i,1:4]=1}
-		}
-		
-	}
-	if(dim(data.sort)[2]==3){
-		x<-data.sort[,1]
-		y<-data.sort[,2]
-		z<-data.sort[,3]
-		liks.tmp <- matrix(0, nb.tip, nl^k)
-		for(i in 1:nb.tip){
-			if(is.na(x[i])){
-				x[i]=2
-				y[i]=2
-				z[i]=2
-			}
-		}
-		for(i in 1:nb.tip){
-			if(x[i]==0 & y[i]==0 & z[i]==0){liks.tmp[i,1]=1}
-			if(x[i]==1 & y[i]==0 & z[i]==0){liks.tmp[i,2]=1}
-			if(x[i]==0 & y[i]==1 & z[i]==0){liks.tmp[i,3]=1}
-			if(x[i]==0 & y[i]==0 & z[i]==1){liks.tmp[i,4]=1}
-			if(x[i]==1 & y[i]==1 & z[i]==0){liks.tmp[i,5]=1}
-			if(x[i]==1 & y[i]==0 & z[i]==1){liks.tmp[i,6]=1}
-			if(x[i]==0 & y[i]==1 & z[i]==1){liks.tmp[i,7]=1}
-			if(x[i]==1 & y[i]==1 & z[i]==1){liks.tmp[i,8]=1}
-			if(x[i]==2 & y[i]==2 & z[i]==2){liks.tmp[i,1:8]=1}
-		}
-	}
-	working.data<-apply(liks.tmp,1,which.max)
+	count.set<-rate.mat.set(phy,data.sort,ntraits,model=model)	
+	working.data<-apply(count.set$liks,1,which.max)
 	counts <- table(working.data)
 	levels <- levels(as.factor(working.data))
 	cols <- as.factor(working.data)
@@ -155,7 +113,7 @@ corDISC<-function(phy, data, ntraits=2, rate.mat=NULL, model=c("ER","SYM","ARD")
 			lower.init = rep(lb, model.set.init$np)
 			upper.init = rep(ub, model.set.init$np)
 			init = nloptr(x0=rep(ip, length.out = model.set.init$np), eval_f=dev.cordisc, lb=lower.init, ub=upper.init, opts=opts, phy=phy,liks=model.set.init$liks,Q=model.set.init$Q,rate=model.set.init$rate,root.p=root.p)
-			cat("Finished. Begin thorough search...", "\n")
+			cat("Finished. Beginning thorough search...", "\n")
 			lower = rep(lb, model.set.final$np)
 			upper = rep(ub, model.set.final$np)	
 			out = nloptr(x0=rep(init$solution, length.out = model.set.final$np), eval_f=dev.cordisc, lb=lower, ub=upper, opts=opts, phy=phy,liks=model.set.final$liks,Q=model.set.final$Q,rate=model.set.final$rate,root.p=root.p)
@@ -164,7 +122,7 @@ corDISC<-function(phy, data, ntraits=2, rate.mat=NULL, model=c("ER","SYM","ARD")
 		}
 		#If a user-specified starting value(s) is supplied:
 		else{
-			cat("Begin subplex optimization routine -- Starting value(s):", ip, "\n")
+			cat("Beginning subplex optimization routine -- Starting value(s):", ip, "\n")
 			opts <- list("algorithm"="NLOPT_LN_SBPLX", "maxeval"="1000000", "ftol_rel"=.Machine$double.eps^0.5)
 			out = nloptr(x0=rep(ip, length.out = model.set.final$np), eval_f=dev.cordisc, lb=lower, ub=upper, opts=opts)
 			loglik <- -out$objective
@@ -344,6 +302,8 @@ rate.mat.set<-function(phy,data.sort,ntraits,model){
 		for(i in 1:nb.tip){
 			if(is.na(x[i])){
 				x[i]=2
+			}
+			if(is.na(y[i])){
 				y[i]=2
 			}
 		}
@@ -352,6 +312,10 @@ rate.mat.set<-function(phy,data.sort,ntraits,model){
 			if(x[i]==0 & y[i]==1){liks[i,2]=1}
 			if(x[i]==1 & y[i]==0){liks[i,3]=1}
 			if(x[i]==1 & y[i]==1){liks[i,4]=1}
+			if(x[i]==2 & y[i]==0){liks[i,c(1,3)]=1}
+			if(x[i]==2 & y[i]==1){liks[i,c(2,4)]=1}
+			if(x[i]==0 & y[i]==2){liks[i,c(1,2)]=1}
+			if(x[i]==1 & y[i]==2){liks[i,c(3,4)]=1}
 			if(x[i]==2 & y[i]==2){liks[i,1:4]=1}
 		}
 	}
@@ -369,7 +333,11 @@ rate.mat.set<-function(phy,data.sort,ntraits,model){
 		for(i in 1:nb.tip){
 			if(is.na(x[i])){
 				x[i]=2
+			}
+			if(is.na(y[i])){
 				y[i]=2
+			}
+			if(is.na(z[i])){
 				z[i]=2
 			}
 		}
@@ -382,11 +350,36 @@ rate.mat.set<-function(phy,data.sort,ntraits,model){
 			if(x[i]==1 & y[i]==0 & z[i]==1){liks[i,6]=1}
 			if(x[i]==0 & y[i]==1 & z[i]==1){liks[i,7]=1}
 			if(x[i]==1 & y[i]==1 & z[i]==1){liks[i,8]=1}
+			#If x is ambiguous but the rest are not:
+			if(x[i]==2 & y[i]==0 & z[i]==0){liks[i,c(1,2)]=1}
+			if(x[i]==2 & y[i]==1 & z[i]==0){liks[i,c(3,5)]=1}
+			if(x[i]==2 & y[i]==0 & z[i]==1){liks[i,c(4,6)]=1}
+			if(x[i]==2 & y[i]==1 & z[i]==1){liks[i,c(7,8)]=1}
+			#If y is ambiguous but the rest are not:
+			if(x[i]==0 & y[i]==2 & z[i]==0){liks[i,c(1,3)]=1}
+			if(x[i]==1 & y[i]==2 & z[i]==0){liks[i,c(2,5)]=1}
+			if(x[i]==0 & y[i]==2 & z[i]==1){liks[i,c(4,7)]=1}
+			if(x[i]==1 & y[i]==2 & z[i]==1){liks[i,c(6,8)]=1}
+			#If z is ambiguous but the rest are not:
+			if(x[i]==0 & y[i]==0 & z[i]==2){liks[i,c(1,4)]=1}
+			if(x[i]==0 & y[i]==1 & z[i]==2){liks[i,c(3,7)]=1}
+			if(x[i]==1 & y[i]==0 & z[i]==2){liks[i,c(2,6)]=1}
+			if(x[i]==1 & y[i]==1 & z[i]==2){liks[i,c(5,8)]=1}
+			#If x and y is ambiguous but z is not:
+			if(x[i]==2 & y[i]==2 & z[i]==0){liks[i,c(1,2,3,5)]=1}
+			if(x[i]==2 & y[i]==2 & z[i]==1){liks[i,c(4,6,7,8)]=1}
+			#If x and z is ambiguous but y is not:
+			if(x[i]==2 & y[i]==0 & z[i]==2){liks[i,c(1,2,4,6)]=1}
+			if(x[i]==2 & y[i]==1 & z[i]==2){liks[i,c(3,5,7,8)]=1}
+			#If y and z is ambiguous but x is not:
+			if(x[i]==0 & y[i]==2 & z[i]==2){liks[i,c(1,3,4,7)]=1}
+			if(x[i]==1 & y[i]==2 & z[i]==2){liks[i,c(2,5,6,8)]=1}
+			#All states are ambiguous:			
 			if(x[i]==2 & y[i]==2 & z[i]==2){liks[i,1:8]=1}
 		}
 	}
 	Q <- matrix(0, nl^k, nl^k)
-	
+
 	obj$np<-max(rate)-1
 	obj$rate<-rate
 	obj$index.matrix<-index.matrix
